"use client";

import { changeOwnPassword } from "@/lib/actions";
import { Card, Text, Heading, Flex, TextField, Button, IconButton, Popover, Link, Tabs, Box, Grid, Dialog, Callout, Tooltip } from "@radix-ui/themes";
import { AlertCircleIcon, CheckIcon, CopyIcon, ExternalLinkIcon, InfoIcon, LockIcon, UserIcon } from "lucide-react";
import { useSession } from "next-auth/react";
import { useEffect, useState } from "react";

// TODO read these from environment
const EMAIL_HOST = "mx1.amogus.cloud";
const SMTP_PORT = "465";
const IMAP_PORT = "993";
const SMTP_SECURITY = "SSL/TLS";
const IMAP_SECURITY = "SSL/TLS";
const WEBMAIL_URL = "https://webmail.amogus.cloud";

export default function SelfService() {
  const session = useSession().data;

  const [newPassword, setNewPassword] = useState("");
  const validPassword = newPassword.length >= 8;
  const [passwordChanged, setPasswordChanged] = useState(false);

  function StaticValueField({ label, value }: { label: string, value: string }) {
    const [checked, setChecked] = useState(false);

    return (
      <>
        <Text weight="light" size="2">{label}</Text>
        <TextField.Root>
          <TextField.Input disabled value={value} />
          <TextField.Slot>
            <IconButton
              size="1"
              variant="ghost"
              onClick={() => {
                navigator.clipboard.writeText(value);
                setChecked(true);
                setTimeout(() => setChecked(false), 1000);
              }}
            >
              {checked
                ? <CheckIcon width={16} height={16} />
                : <CopyIcon width={16} height={16} />
              }
            </IconButton>
          </TextField.Slot>
        </TextField.Root>
      </>
    );
  }

  function PasswordChangedAlert() {
    const [hide, setHide] = useState(false);

    useEffect(() => {
      let timeout = setTimeout(() => {
        setHide(true);
        setTimeout(() => setPasswordChanged(false), 500);
      }, 5000);
      return () => clearTimeout(timeout);
    });

    return (
      <Callout.Root className={`transition-all ${hide ? "opacity-0 h-0 m-0 !p-0" : " mb-4"}`}>
        <Callout.Icon><CheckIcon /></Callout.Icon>
        <Callout.Text>Password updated!</Callout.Text>
      </Callout.Root>
    );
  }

  return (
    <>
      {passwordChanged && (
        <PasswordChangedAlert />
      )}

      <Heading className="pb-4">Account settings</Heading>

      <Grid display="inline-grid" rows="1" columns="2" gap="4">
        <Card className="h-fit">
          <Heading size="3">Credentials</Heading>
          <Text weight="light" size="2">These are the details you can use to authenticate via SMTP and IMAP.</Text>

          <Flex direction="column" gap="3" className="pt-2">
            <TextField.Root>
              <TextField.Slot>
                <UserIcon height="16" width="16" />
              </TextField.Slot>
              <TextField.Input disabled value={session?.user?.email ?? 'n/a'} />
              <TextField.Slot>
                <Popover.Root>
                  <Popover.Trigger>
                    <IconButton asChild size="1" variant="ghost">
                      <InfoIcon height="16" width="16" />
                    </IconButton>
                  </Popover.Trigger>
                  <Popover.Content className="w-96">
                    <Text>
                      Your username uniquely identifies your account and therefore cannot be changed. If you need a different email address, you can <Link onClick={() => alert("todo")}>request an alias</Link> instead.
                    </Text>
                  </Popover.Content>
                </Popover.Root>
              </TextField.Slot>
            </TextField.Root>

            <Flex direction="row" gap="3">
              <TextField.Root className="flex-1">
                <TextField.Slot>
                  <LockIcon height="16" width="16" />
                </TextField.Slot>
                <TextField.Input disabled value={"\u25CF".repeat(10)} />
              </TextField.Root>

              <Dialog.Root>
                <Dialog.Trigger>
                  <Button variant="outline">
                    Change
                  </Button>
                </Dialog.Trigger>

                <Dialog.Content>
                  <Dialog.Title>Update password</Dialog.Title>

                  <Box pb="4">
                    <Callout.Root>
                      <Callout.Icon>
                        <AlertCircleIcon />
                      </Callout.Icon>
                      <Callout.Text>You will be logged out of every configured mail client!</Callout.Text>
                    </Callout.Root>
                  </Box>

                  <TextField.Root>
                    <TextField.Slot>
                      <LockIcon height="16" width="16" />
                    </TextField.Slot>
                    <TextField.Input
                      type="password"
                      placeholder="New password"
                      value={newPassword}
                      onChange={(e) => setNewPassword(e.currentTarget.value)}
                    />
                  </TextField.Root>

                  <Flex gap="3" mt="4" justify="end">
                    <Dialog.Close>
                      <Button variant="outline" onClick={() => setNewPassword("")}>Cancel</Button>
                    </Dialog.Close>
                    <Dialog.Close>
                      {validPassword
                        ? (
                          <Button
                          variant="solid"
                          onClick={async () => {
                            setNewPassword("");

                            try {
                              setPasswordChanged(false);
                              await changeOwnPassword(newPassword);
                              setPasswordChanged(true);
                            } catch(e) {
                              alert(e);
                            }
                          }}
                        >Update</Button>
                        )
                        : (
                          <Tooltip content="The password must have a minimum length of 8 characters">
                            <Button variant="solid" disabled>Update</Button>
                          </Tooltip>
                        )
                      }
                    </Dialog.Close>
                  </Flex>
                </Dialog.Content>
              </Dialog.Root>
            </Flex>
          </Flex>
        </Card>

        <Card className="h-fit">
          <Heading size="3">Server configuration</Heading>
          <Text weight="light" size="2">You can use these details to log in using any mail client.</Text>

          <Box pt="2">
            <Tabs.Root defaultValue="smtp">
              <Tabs.List size="2">
                <Tabs.Trigger value="smtp">Outgoing</Tabs.Trigger>
                <Tabs.Trigger value="imap">Incoming</Tabs.Trigger>
                <Flex align="center" justify="end" grow="1" pr="3">
                  <Link href={WEBMAIL_URL} target="_blank">
                    <Button variant="ghost">Webmail <ExternalLinkIcon size="14" /></Button>
                  </Link>
                </Flex>
              </Tabs.List>

              <Box pt="3" pb="2">
                <Tabs.Content value="smtp">
                  <StaticValueField label="SMTP Server" value={EMAIL_HOST} />
                  <StaticValueField label="SMTP Port" value={SMTP_PORT} />
                  <StaticValueField label="Connection security" value={SMTP_SECURITY} />
                </Tabs.Content>
                <Tabs.Content value="imap">
                  <StaticValueField label="IMAP Server" value={EMAIL_HOST} />
                  <StaticValueField label="IMAP Port" value={IMAP_PORT} />
                  <StaticValueField label="Connection security" value={IMAP_SECURITY} />
                </Tabs.Content>
              </Box>
            </Tabs.Root>
          </Box>
        </Card>
      </Grid>
    </>
  );
}
